@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: bootstrap.sh 71 2005-01-16 14:23:22Z lennart $
+# $Id: bootstrap.sh 82 2005-08-05 23:51:50Z lennart $
 
 # This file is part of nss-mdns.
 #
@@ -17,6 +17,8 @@
 # along with nss-mdns; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 
+VERSION=1.9
+
 run_versioned() {
     local P
     type -p "$1-$2" &> /dev/null && P="$1-$2" || local P="$1"
@@ -27,7 +29,7 @@ run_versioned() {
 
 if [ "x$1" = "xam" ] ; then
     set -ex
-    run_versioned automake 1.8 -a -c --foreign
+    run_versioned automake "$VERSION" -a -c --foreign
     ./config.status
 else 
     set -ex
@@ -35,13 +37,13 @@ else
     rm -rf autom4te.cache
     rm -f config.cache
 
-    run_versioned aclocal 1.8
+    run_versioned aclocal "$VERSION"
     libtoolize -c --force
     autoheader
-    run_versioned automake 1.8 -a -c --foreign
+    run_versioned automake "$VERSION" -a -c --foreign
     autoconf -Wall
 
-    CFLAGS="-g -O0" ./configure --sysconfdir=/etc "$@"
+    CFLAGS="$CFLAGS -g -O0" ./configure --sysconfdir=/etc --localstatedir=/var "$@"
 
     make clean
 fi
