@@ -1,4 +1,4 @@
-/* $Id: query.c 82 2005-08-05 23:51:50Z lennart $ */
+/* $Id: query.c 114 2007-02-08 16:42:22Z lennart $ */
 
 /***
   This file is part of nss-mdns.
@@ -73,46 +73,55 @@ int mdns_open_socket(void) {
     struct sockaddr_in sa;
     int fd = -1, ttl, yes;
 
-    if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
-        fprintf(stderr, "socket() failed: %s\n", strerror(errno));
+    if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
         goto fail;
-    }
 
     ttl = 255;
-    if (setsockopt(fd, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl)) < 0) {
-        fprintf(stderr, "IP_MULTICAST_TTL failed: %s\n", strerror(errno));
+    if (setsockopt(fd, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl)) < 0)
         goto fail;
-    }
 
     sa.sin_family = AF_INET;
     sa.sin_port = 0;
     sa.sin_addr.s_addr = INADDR_ANY;
     
-    if (bind(fd, (struct sockaddr*) &sa, sizeof(sa)) < 0) {
-        fprintf(stderr, "bind() failed: %s\n", strerror(errno));
+    if (bind(fd, (struct sockaddr*) &sa, sizeof(sa)) < 0)
         goto fail;
-    }
     
+#ifdef IP_PKTINFO
     yes = 1;
-    if (setsockopt(fd, IPPROTO_IP, IP_RECVTTL, &yes, sizeof(yes)) < 0) {
-        fprintf(stderr, "IP_RECVTTL failed: %s\n", strerror(errno));
+    if (setsockopt(fd, IPPROTO_IP, IP_PKTINFO, &yes, sizeof(yes)) < 0)
         goto fail;
-    }
+#else
+    
+#ifdef IP_RECVINTERFACE
+    yes = 1;
+    if (setsockopt (fd, IPPROTO_IP, IP_RECVINTERFACE, &yes, sizeof(yes)) < 0)
+      goto fail;
+#elif defined(IP_RECVIF)
+    yes = 1;
+    if (setsockopt (fd, IPPROTO_IP, IP_RECVIF, &yes, sizeof(yes)) < 0)
+        goto fail;
+#endif
+    
+#ifdef IP_RECVDSTADDR
+    yes = 1;
+    if (setsockopt (fd, IPPROTO_IP, IP_RECVDSTADDR, &yes, sizeof(yes)) < 0)
+        goto fail;
+#endif
+    
+#endif /* IP_PKTINFO */
+    
+#ifdef IP_RECVTTL
+    yes = 1;
+    if (setsockopt(fd, IPPROTO_IP, IP_RECVTTL, &yes, sizeof(yes)) < 0)
+        goto fail;
+#endif
 
-    if (setsockopt(fd, IPPROTO_IP, IP_PKTINFO, &yes, sizeof(yes)) < 0) {
-        fprintf(stderr, "IP_PKTINFO failed: %s\n", strerror(errno));
+    if (set_cloexec(fd) < 0)
         goto fail;
-    }
     
-    if (set_cloexec(fd) < 0) {
-        fprintf(stderr, "FD_CLOEXEC failed: %s\n", strerror(errno));
+    if (set_nonblock(fd) < 0)
         goto fail;
-    }
-    
-    if (set_nonblock(fd) < 0) {
-        fprintf(stderr, "O_ONONBLOCK failed: %s\n", strerror(errno));
-        goto fail;
-    }
 
     return fd;
 
@@ -127,13 +136,20 @@ static int send_dns_packet(int fd, struct dns_packet *p) {
     struct sockaddr_in sa;
     struct msghdr msg;
     struct iovec io;
+#ifdef IP_PKTINFO
     struct cmsghdr *cmsg;
+    uint8_t cmsg_data[CMSG_SPACE(sizeof(struct in_pktinfo))];
     struct in_pktinfo *pkti;
-    uint8_t cmsg_data[sizeof(struct cmsghdr) + sizeof(struct in_pktinfo)];
+#elif defined(IP_SENDSRCADDR)
+    struct cmsghdr *cmsg;
+    uint8_t cmsg_data[CMSG_SPACE(sizeof(struct in_addr))];
+    struct in_addr *addr;
+#endif
     int i, n;
     struct ifreq ifreq[32];
     struct ifconf ifconf;
     int n_sent = 0;
+    int last_index = -1;
 
     assert(fd >= 0 && p);
     assert(dns_packet_check_valid(p) >= 0);
@@ -144,31 +160,48 @@ static int send_dns_packet(int fd, struct dns_packet *p) {
     io.iov_base = p->data;
     io.iov_len = p->size;
 
-    memset(cmsg_data, 0, sizeof(cmsg_data));
-    cmsg = (struct cmsghdr*) cmsg_data;
-    cmsg->cmsg_len = sizeof(cmsg_data);
-    cmsg->cmsg_level = IPPROTO_IP;
-    cmsg->cmsg_type = IP_PKTINFO;
-
-    pkti = (struct in_pktinfo*) (cmsg_data + sizeof(struct cmsghdr));
-    pkti->ipi_ifindex = 0;
-    
     memset(&msg, 0, sizeof(msg));
     msg.msg_name = &sa;
     msg.msg_namelen = sizeof(sa);
     msg.msg_iov = &io;
     msg.msg_iovlen = 1;
-    msg.msg_control = cmsg_data;
-    msg.msg_controllen = sizeof(cmsg_data);
     msg.msg_flags = 0;
 
+#ifdef IP_PKTINFO
+    memset(cmsg_data, 0, sizeof(cmsg_data));
+    msg.msg_control = cmsg_data;
+    msg.msg_controllen = sizeof(cmsg_data);
+	
+    cmsg = CMSG_FIRSTHDR(&msg);
+    cmsg->cmsg_len = CMSG_LEN(sizeof(struct in_pktinfo));
+    cmsg->cmsg_level = IPPROTO_IP;
+    cmsg->cmsg_type = IP_PKTINFO;
+
+    pkti = (struct in_pktinfo*) CMSG_DATA(cmsg);
+
+    msg.msg_controllen = cmsg->cmsg_len;
+#elif defined(IP_SENDSRCADDR)
+    memset(cmsg_data, 0, sizeof(cmsg_data));
+    msg.msg_control = cmsg_data;
+    msg.msg_controllen = sizeof(cmsg_data);
+        
+    cmsg = CMSG_FIRSTHDR(&msg);
+    cmsg->cmsg_len = CMSG_LEN(sizeof(struct in_addr));
+    cmsg->cmsg_level = IPPROTO_IP;
+    cmsg->cmsg_type = IP_SENDSRCADDR;
+	
+    addr = (struct in_addr *)CMSG_DATA(cmsg);
+
+    msg.msg_controllen = cmsg->cmsg_len;
+#elif defined(__GNUC__)
+#warning "FIXME: We need some code to set the outgoing interface/local address here if IP_PKTINFO/IP_SENDSRCADDR is not available"
+#endif
+   
     ifconf.ifc_req = ifreq;
     ifconf.ifc_len = sizeof(ifreq);
     
-    if (ioctl(fd, SIOCGIFCONF, &ifconf) < 0) {
-        fprintf(stderr, "SIOCGIFCONF failed: %s\n", strerror(errno));
+    if (ioctl(fd, SIOCGIFCONF, &ifconf) < 0)
         return -1;
-    }
 
     for (i = 0, n = ifconf.ifc_len/sizeof(struct ifreq); i < n; i++) {
         struct sockaddr_in *ifsa;
@@ -197,18 +230,28 @@ static int send_dns_packet(int fd, struct dns_packet *p) {
 
         if (ioctl(fd, SIOCGIFINDEX, &ifreq[i]) < 0) 
             continue; /* See above why we ignore this error */
-        
-        pkti->ipi_ifindex = ifreq[i].ifr_ifindex;
+
+#ifdef IP_PKTINFO
+        /* Only send the the packet once per interface. We assume that
+         * multiple addresses assigned to the same interface follow
+         * immediately one after the other.*/
+        if (last_index == ifreq[i].ifr_ifindex)
+            continue;
+
+        last_index = pkti->ipi_ifindex = ifreq[i].ifr_ifindex;
+#elif defined(IP_SENDSRCADDR)
+	addr->s_addr = ifsa->sin_addr.s_addr;
+#elif defined(__GNUC__)
+#warning "FIXME: We need some code to set the outgoing interface/local address here if IP_PKTINFO/IP_SENDSRCADDR is not available"
+#endif
         
         for (;;) {
             
             if (sendmsg(fd, &msg, MSG_DONTROUTE) >= 0)
                 break;
 
-            if (errno != EAGAIN) {
-                fprintf(stderr, "sendmsg() failed: %s\n", strerror(errno));
+            if (errno != EAGAIN)
                 return -1;
-            }
             
             if (wait_for_write(fd, NULL) < 0)
                 return -1;
@@ -228,7 +271,8 @@ static int recv_dns_packet(int fd, struct dns_packet **ret_packet, uint8_t *ret_
     uint8_t aux[64];
     assert(fd >= 0);
 
-    p = dns_packet_new();
+    if (!(p = dns_packet_new()))
+        return -1; /* OOM */
 
     io.iov_base = p->data;
     io.iov_len = sizeof(p->data);
@@ -251,16 +295,19 @@ static int recv_dns_packet(int fd, struct dns_packet **ret_packet, uint8_t *ret_
             *ret_ttl = 0;
             
             for (cmsg = CMSG_FIRSTHDR(&msg); cmsg != NULL; cmsg = CMSG_NXTHDR(&msg,cmsg)) {
-                if (cmsg->cmsg_level == SOL_IP && cmsg->cmsg_type == IP_TTL) {
+#ifdef SOL_IP
+                if (cmsg->cmsg_level == SOL_IP && cmsg->cmsg_type == IP_TTL)
+#else
+                if (cmsg->cmsg_level == IPPROTO_IP && cmsg->cmsg_type == IP_TTL)
+#endif
+		{
                     *ret_ttl = (uint8_t) (*(uint32_t*) CMSG_DATA(cmsg));
                     break;
                 }
             }
                      
-            if (cmsg == NULL) {
-                fprintf(stderr, "Didn't recieve TTL\n");
+            if (!cmsg)
                 goto fail;
-            }
 
             p->size = (size_t) l;
 
@@ -268,10 +315,8 @@ static int recv_dns_packet(int fd, struct dns_packet **ret_packet, uint8_t *ret_
             return 0;
         }
 
-        if (errno != EAGAIN) {
-            fprintf(stderr, "recvfrom() failed: %s\n", strerror(errno));
+        if (errno != EAGAIN)
             goto fail;
-        }
         
         if ((r = wait_for_read(fd, end)) < 0)
             goto fail;
@@ -296,21 +341,17 @@ static int send_name_query(int fd, const char *name, uint16_t id, int query_ipv4
 
     assert(fd >= 0 && name && (query_ipv4 || query_ipv6));
 
-    if (!(p = dns_packet_new())) {
-        fprintf(stderr, "Failed to allocate DNS packet.\n");
-        goto finish;
-    }
+    if (!(p = dns_packet_new()))
+        goto finish; /* OOM */
 
     dns_packet_set_field(p, DNS_FIELD_ID, id);
     dns_packet_set_field(p, DNS_FIELD_FLAGS, DNS_FLAGS(0, 0, 0, 0, 0, 0, 0, 0, 0, 0));
 
 #ifndef NSS_IP6_ONLY
     if (query_ipv4) {
-        if (!(prev_name = dns_packet_append_name(p, name))) {
-            fprintf(stderr, "Bad host name\n");
-            goto finish;
-            
-        }
+        if (!(prev_name = dns_packet_append_name(p, name)))
+            goto finish; /* Bad name */
+
         dns_packet_append_uint16(p, DNS_TYPE_A);
         dns_packet_append_uint16(p, DNS_CLASS_IN);
         qdcount++;
@@ -319,10 +360,8 @@ static int send_name_query(int fd, const char *name, uint16_t id, int query_ipv4
     
 #ifndef NSS_IP4_ONLY
     if (query_ipv6) {
-        if (!dns_packet_append_name_compressed(p, name, prev_name)) {
-            fprintf(stderr, "Bad host name\n");
-            goto finish;
-        }
+        if (!dns_packet_append_name_compressed(p, name, prev_name))
+            goto finish; /* Bad name */
         
         dns_packet_append_uint16(p, DNS_TYPE_AAAA);
         dns_packet_append_uint16(p, DNS_CLASS_IN);
@@ -510,18 +549,14 @@ static int send_reverse_query(int fd, const char *name, uint16_t id) {
 
     assert(fd >= 0 && name);
 
-    if (!(p = dns_packet_new())) {
-        fprintf(stderr, "Failed to allocate DNS packet.\n");
+    if (!(p = dns_packet_new()))
         goto finish;
-    }
 
     dns_packet_set_field(p, DNS_FIELD_ID, id);
     dns_packet_set_field(p, DNS_FIELD_FLAGS, DNS_FLAGS(0, 0, 0, 0, 0, 0, 0, 0, 0, 0));
 
-    if (!dns_packet_append_name(p, name)) {
-        fprintf(stderr, "Bad host name\n");
+    if (!dns_packet_append_name(p, name))
         goto finish;
-    }
 
     dns_packet_append_uint16(p, DNS_TYPE_PTR);
     dns_packet_append_uint16(p, DNS_CLASS_IN);
@@ -678,6 +713,4 @@ int mdns_query_ipv6(int fd, const ipv6_address_t *ipv6, void (*name_func)(const
     
     return query_reverse(fd, name, name_func, userdata);
 }
-
-
 #endif
